/******************************************************************************
 * g3DGMV
 * Copyright (C) <1999> <Nick Yost>
 *-----------------------------------------------------------------------------
 *  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ******************************************************************************/

#include <gdk_imlib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <math.h>
#include <gdk/gdkkeysyms.h>
#include <stdlib.h>
#include <stdio.h>
#include "draw_entry.h"

// color table
static GdkGC *GCTable[1000];
static gint maxpin=0; 
static gint FrameWidth=100, FrameHeight=100;
static GdkPoint FrameCenter;
static gint Feet=1, Meters=2, Pixels=4;
static gint ColorStartPosition = 0;
static gint WaterLevel = 0;
//static gint Radians=0, Feet=1, ArcSeconds=3;

/*---------------------- Map house keeping routines ------------------------- */

/******************************************************************************
 * Name:         gtk_create_Elevmap
 *-----------------------------------------------------------------------------
 * Description: create and Initialize the map structure. 
 *  
 *-----------------------------------------------------------------------------
 * Parameters:
 *    None
 * Returns:
 *    ElevMap - Elevation map
 ******************************************************************************/
ElevMap *gtk_create_Elevmap()
{
	ElevMap *Map;

	Map = g_malloc(sizeof(ElevMap));
	Map->NWXPOS=0.0;
	Map->NWYPOS=0.0;
	Map->SWXPOS=0.0; 
	Map->SWYPOS=0.0;
	Map->SEXPOS=0.0;
	Map->SEYPOS=0.0;
	Map->NEXPOS=0.0;
	Map->NEYPOS=0.0;
	Map->GUnit=0; // defines the graphical format of the above
	Map->ElvUint=0; // define the garphicl format of elevation
	Map->minelev=0; // min elevation
	Map->MaxElev=0; // max elevation
	Map->spatial_resolution_x=0.0;// number of pixels per gunit in x
	Map->spatial_resolution_y=0.0;// number of pixels per gunit in y
	Map->spatial_resolution_z=0.0;// number of pixels per gunit in z
	Map->col_list_ptr = (col_list *)NULL;
	Map->pixmap =(GdkPixmap*)NULL;
	Map->pixmap_size.x=0;
	Map->pixmap_size.y=0;
	Map->pbar=(GtkWidget *)NULL;
	Map->background= NULL;
	strcpy(Map->bg_str, " "); 
	Map->bk_enable= FALSE;
	return (Map);
}

/******************************************************************************
 * Name:         destroy_map
 *-----------------------------------------------------------------------------
 * Description: distroys the map by removing the structure and bitmap used .  
 *              by it. 
 *-----------------------------------------------------------------------------
 * Parameters:
 *    col_list_ptr - pointer to a link list that contains all data points on
 *                   the map.
 ******************************************************************************/
col_list *destroy_map(col_list *col_list_ptr )
{   
	int columns, i;

	columns= col_list_ptr[0].lsize;
    
	for (i=0; i <= columns; i++)
	{
		g_free(col_list_ptr[i].ptr);
	}
	g_free(col_list_ptr);
      
	return ((col_list *)NULL);
}

/******************************************************************************
 * Name:         col_count_check
 *-----------------------------------------------------------------------------
 * Description: checks to see if the column count is at one of the ends of  
 *              fmaxelev of a column (top or bottom). The test is based on the  
 *              direction of the count.
 *-----------------------------------------------------------------------------
 * Parameters:
 *    invcnt - direct of count
 *    col_count - column count position
 *    columns - total number of columns
 ******************************************************************************/
gboolean col_count_check(gboolean invcnt,int col_count,int columns)
{ 
	if (invcnt)
		return ( col_count <= 1);
	else
		return (col_count >= columns-1);
}

/******************************************************************************
 * Name:         row_count_check
 *-----------------------------------------------------------------------------
 * Description:  checks to see if the row count is at one of the ends of 
 *               of a row (right or left). The test is based on the direction 
 *               of the count
 *-----------------------------------------------------------------------------
 * Parameters:
 *    invcnt - direct of count
 *    row_count - row count position
 *    rows - total number of rows
 *    count - 
 ******************************************************************************/
gboolean row_count_check(gboolean invcnt,int row_count,int rows, int count)
{ 
	if (invcnt)
		return (row_count > 0  && count != 0); 
	else
		return (row_count < rows  && count != 0); 
}

/******************************************************************************
 * Name:         Resize_pixmap
 *-----------------------------------------------------------------------------
 * Description:  Size the pixmap to fit either a normal map or a rotated
 *               map
 *-----------------------------------------------------------------------------
 * Parameters:
 *    Map - A Elevation map
 *    drawable- the GdkDrawable area
 *    noraml  - craete a normal size map 
 ******************************************************************************/
GdkPoint Resize_pixmap(ElevMap *map,GdkDrawable *drawable,gboolean normal)
{ 
  int iOffset = 0 ;
  GdkPoint offset;
  gint columns= map->col_list_ptr[0].lsize;
  gint rows= map->col_list_ptr[0].ptr[0].z;

  offset.x = 0;
  offset.y = 0;

  if (normal)
	{
		iOffset=rows;
		map->pixmap_size.x=columns;
		map->pixmap_size.y=rows;
	}
    else
	{
		iOffset = sqrt((columns*columns)+(rows*rows));
		offset.x = iOffset-columns;
		offset.y = iOffset-rows;
		map->pixmap_size.x=iOffset;
		map->pixmap_size.y=iOffset;
	}

    if(map->background != NULL && map->bk_enable)
      {
	gdk_imlib_render(map->background,iOffset,iOffset);  
	map->pixmap = gdk_imlib_move_image(map->background);
      }
    else if (normal)
      map->pixmap = gdk_pixmap_new(drawable,
				    columns,
				    rows,
				    -1); 
    else 
      map->pixmap = gdk_pixmap_new(drawable,
				   iOffset,
				   iOffset,
				   -1);
    return offset;
}

/******************************************************************************
 * Name:         Rotation
 *-----------------------------------------------------------------------------
 * Description: The following routine is used to calculates a simulated 
 *              3D perspective for x and y for a points with a x,y,z cordinates 
 *              around one or more selected axis. It will calculate the z 
 *              value from the elevtion value. 
 *  
 *-----------------------------------------------------------------------------
 * Parameters:
 *    ElevMap - Elevation map
 *    x - x position
 *    y - y position
 *    elev - Map elevation
 *    cAx - cos of the angle x axis
 *    cAy - cos of the angle y axis
 *    cAz - cos of the angle y axis
 *    sAx - sin of the angle x axis
 *    sAy - sin of the angle y axis
 *    sAz - sin of the angle y axis
 * Return
 *    x and y for a points
 ******************************************************************************/
GdkPoint Rotation(ElevMap *map,int x, int y, int elev,
			  double cAx,double cAy,double cAz,
			  double sAx,double sAy,double sAz)
{
	double z;
	double dX,dY,dZ,tdX,tdY,tdZ;
	GdkPoint pos;


	//This calculates the z value. Basically this scale the elevation 
	//to match the pixel resolution of x and y
	if( map->ElvUint == Meters && map->GUnit==Meters)
		z=(((elev+map->minelev)*map->spatial_resolution_z) -
		map->minelev)/map->spatial_resolution_x;
	else if( map->ElvUint == Feet && map->GUnit==Meters)
		z=elev/(map->spatial_resolution_x*3.28);
	else if( map->ElvUint == Pixels)
		z=elev;
	else
		z=elev/(map->spatial_resolution_x*18);
	

	// now calculates a simulated 3D perspective for x and y.
	dX=(double)x;
	dY=(double)y;
	dZ= z;

	//rotate along z axis
	tdX=dX*cAx-dY*sAx;
	tdY=dX*sAx+dY*cAx;
	dX = tdX; dY = tdY;

	//rotation along y axis
	tdX = dX *cAz + dZ * sAz;
	tdZ = -dX * sAz+ dZ * cAz;
	dX = tdX; dZ = tdZ;

	//rotation along x axis
	tdY = dY*cAy- dZ * sAy;
	dY = tdY;

	pos.x = (int)dX;
	pos.y = (int)dY;
	
	return pos;
}

/*----------------Graphical content and color routines for Map--------------- */

/******************************************************************************
 * Name:         gtk_Generate_Image2
 *-----------------------------------------------------------------------------
 * Description: This routine draws a line or solid map onto the bitmap. The
 *              the bitmap can be drawn rotated or flat.
 *  
 *-----------------------------------------------------------------------------
 * Parameters: 
 *    Map - A Elevation map
 *    drawable- the GdkDrawable area
 *    ty - type of map to draw
 *    xx - x axis angle
 *    yy - y axis angle
 *    zz - z axis angle
 ******************************************************************************/
void gtk_Generate_Image(ElevMap *map,GdkDrawable *drawable,
			int ty,double xx,double yy,double zz)
{
	col_list *col_list_ptr=map->col_list_ptr;
	double pvalue2=0.0;
	int columns=0.0;
	int x_temp=0,y_temp=0,elev=0;
	int rows=0.0;
	GdkPoint offset;
	//int offset = 0,offsetx =0,offsety =0 ;
	double x,y,z,xp,yp,zp;
	int done = FALSE;
	int count=1;
	double pvalue=0.0; 
	float pv=0.0;
	int type,col_count,row_count=0, gc_com;
	double Ax,Ay,Az;
	gboolean invccnt=FALSE,invrcnt=TRUE;
 
	pvalue =0.0;
    Ax=xx;Ay=yy;Az=zz;type=(ty & 0x00ff);
    if (Ax < 0) Ax=360+Ax;
	if (Ay < 0) Ay=360+Ay;
    invccnt =FALSE;invrcnt=TRUE;
    if (Ax >90) {invccnt =FALSE;invrcnt=FALSE;}
    if (Ax > 180){invccnt =TRUE; invrcnt=FALSE;}
    if (Ax > 270){invccnt =TRUE; invrcnt= TRUE;}

    Ax=Ax*.01745; Ay=Ay*.01745; Az=Az*.01745;
    columns= col_list_ptr[0].lsize;
    rows= col_list_ptr[0].ptr[0].z;
    
    if (invccnt)
      col_count=columns-1;
    else
      col_count=1;
 
    pv=1/(float)rows;
    done = FALSE;
  
    offset=Resize_pixmap(map,drawable,(xx==0 && yy==0 && zz==0));
      

   
	{ 

		/* Start of map drawing loop */
		while(!done)
		{
			if(col_count)
			{
				count = (rows*columns)/100; 
				if (invrcnt)
					row_count = col_list_ptr[col_count].lsize-1;
				else
					row_count=0; 
				while (row_count_check(invrcnt,row_count,
					col_list_ptr[col_count].lsize,count))
				{
					pvalue2=1-( (columns-pvalue)/(float)columns);
					pvalue+=pv; 
					if (type == 1 || type == 3)
					{
						x=col_count-(columns/2);
						y= (col_list_ptr[col_count].row_offset-row_count)-(rows/2);
						//z=col_list_ptr[col_count].ptr[row_count].z;
						elev=col_list_ptr[col_count].ptr[row_count].z;
						if( map->ElvUint == 2 && map->GUnit==2)
							z=(((elev+map->minelev)*map->spatial_resolution_z) -
							map->minelev)/map->spatial_resolution_x;
						else if( map->ElvUint == 1 && map->GUnit==2)
							z=elev/(map->spatial_resolution_x*3.28);
						else if( map->ElvUint == 4)
							z=elev;
						else
							z=elev/(map->spatial_resolution_x*18);
						
						//rotate along z axis
						xp=x*cos(Ax)-y*sin(Ax);
						yp=x*sin(Ax)+y*cos(Ax);
						y=yp; x=xp;
						//rotation along y axis
						xp =x *cos(Az) + z * sin(Az);
						zp = -x * sin(Az)+ z * cos(Az);
						z=zp; x=xp;
						//rotation along x axis
						yp=y*cos(Ay)- z * sin(Ay);
						zp=x*sin(Ay)+ z * cos(Ay);
						z=zp; y=yp;
						x =x+(columns/2); y=y+(rows/2);

						gc_com=col_list_ptr[col_count].ptr[row_count].gc;
						if((ty & 0xff00) != 0 && row_count > 0)
						{
							if(gc_com >= 256 && type == 3)
							{
							  gdk_draw_point(map->pixmap,
									 GetPen(drawable,
										col_list_ptr[col_count].ptr[row_count].gc)
									 , x+(offset.x/2),y+(offset.y/2));
                                                        }
							else if(type != 3 && gc_com >= 0)
							{
							  if( elev < WaterLevel)
							  {
							    if(WaterLevel > 0 )gc_com=1 ;else gc_com=0;
							    gdk_draw_point(map->pixmap,
								GetPen(drawable,
								col_list_ptr[col_count].ptr[row_count].gc+50+gc_com)
								, x+(offset.x/2),y+(offset.y/2));
							  }
							  else
								gdk_draw_point(map->pixmap,
								GetPen(drawable,
								col_list_ptr[col_count].ptr[row_count].gc+ColorStartPosition)
								, x+(offset.x/2),y+(offset.y/2));
							}
						}
						else if(row_count > 0)
						{
							if(gc_com >= 256 && type == 3)
						        {
							  gdk_draw_point(map->pixmap,
									 GetPen(drawable,
										col_list_ptr[col_count].ptr[row_count].gc)
									 , x+(offset.x/2),y+(offset.y/2));
							}
							else if (type != 3 && gc_com >= 0)
						        {

							  if( elev < WaterLevel)
							  {
							    if(WaterLevel > 0 )gc_com=1 ;else gc_com=0;
							    gdk_draw_line  (map->pixmap,
								GetPen(drawable,
								col_list_ptr[col_count].ptr[row_count].gc+50+gc_com),
								x+(offset.x/2),y+(offset.y/2),
								x+(offset.x/2),y+(offset.y/2)-3);
							  }
							  else
								gdk_draw_line  (map->pixmap,
								GetPen(drawable,
								col_list_ptr[col_count].ptr[row_count].gc+ColorStartPosition),
								x+(offset.x/2),y+(offset.y/2),
								x+(offset.x/2),y+(offset.y/2)-3);
							}
								
						}
						x_temp=x; y_temp=y;

					}
					else if (type == 2) 
					{ 
						x=col_count;
						y= col_list_ptr[col_count].row_offset-row_count;
						gc_com=col_list_ptr[col_count].ptr[row_count].gc;
						if(gc_com >= 256)
							gdk_draw_point(map->pixmap,
							GetPen(drawable,
							col_list_ptr[col_count].ptr[row_count].gc)
							,x-1,y);
					}
					else 
					{ 
						x=col_count;
						y= col_list_ptr[col_count].row_offset-row_count;
						gc_com=col_list_ptr[col_count].ptr[row_count].gc;
						elev=col_list_ptr[col_count].ptr[row_count].z;
						if(gc_com >= 0)
						{
						  if( elev < WaterLevel)
						  {
						    if(WaterLevel > 0 )gc_com=1 ;else gc_com=0;
						    gdk_draw_point(map->pixmap,
							GetPen(drawable,
							col_list_ptr[col_count].ptr[row_count].gc+50+gc_com)
							,x-1,y);
						  }
						  else
							gdk_draw_point(map->pixmap,
							GetPen(drawable,
							col_list_ptr[col_count].ptr[row_count].gc+ColorStartPosition)
							,x-1,y);
						}
					}

					count--; 

					if (invrcnt) 
						row_count--; 
					else 
						row_count++;
				}
       
       
				if(map->pbar != NULL && (ty & 0xff00) == 0)
				{
					gtk_progress_bar_update(GTK_PROGRESS_BAR (map->pbar),pvalue2);
					while (gtk_events_pending())
						gtk_main_iteration();
				}
     
  
				if (col_count_check(invccnt,col_count,columns))
				{
					done =TRUE;pvalue2 =0.0;col_count=0;
				}
				else
				{ 
					if((ty & 0xff00) != 0 && col_count <= 10 && invccnt )
						col_count=1; 
					else if((ty & 0xff00) != 0 &&
						!col_count_check(invccnt,col_count,columns-10))
					{
						if (invccnt) 
							col_count=col_count-10;
						else 
							col_count=col_count+10; 
					}
					else if((ty & 0xff00) != 0 && 
						col_count != col_list_ptr[0].lsize-1 && !invccnt)
						col_count=col_list_ptr[0].lsize-1;
					else 
					{
						if (invccnt) col_count--;else col_count++;
					}
				}
      }
     }
    }

    if (done)
	{
		pv=1/(float)rows;
		pvalue =0.0;
		if(map->pbar != NULL)
		{
			gtk_progress_bar_update(GTK_PROGRESS_BAR (map->pbar),pvalue2);
			while (gtk_events_pending())
				gtk_main_iteration();
		}
    }
    
}

/******************************************************************************
 * Name:         Generate_Wire_frame_Image 
 *-----------------------------------------------------------------------------
 * Description: This routine draws a wire frame or solid modal map onto 
 *              the bitmap.
 *-----------------------------------------------------------------------------
 * Parameters: 
 *    ElevMap - Elevation map
 *    hdc- Handle to a Device Context
 *    LeftOffset - offset from Left side of map
 *    TopOffset - offset from top of map
 *    Width - Width of area from LeftOffset
 *    Height - Height of area from TopOffset
 *    Step - The step size between map points
 *    xx - angle to rotate point around x axis
 *    yy - angle to rotate point around y axis 
 *    zz - angle to rotate point around z axis 
 *    FrameMode - set True to be wire frame or False for solid model
 *    MapColors - set True to use Map colors or False for other source
 *    zoom - set the magnification value
 ******************************************************************************/
void Generate_Wire_frame(ElevMap *map, GdkDrawable *drawable,
			 int LeftOffset, int TopOffset, int Width, int Height,int Step,
			 double xx,double yy,double zz, gboolean FrameMode, 
			 gboolean MapColors, float zoom)
{
	col_list *col_list_ptr=map->col_list_ptr;
	int columns=0;
	int rows=0;
	GdkGC *gc;
	GdkPoint pos,MeshOffset,LastPos[1000];
	GdkPoint pt[4];
	int index=0;
	double cAx,cAy,cAz,sAx,sAy,sAz;
	int done = FALSE;
	int gc_com =0;
	int col_count,row_count=0;
	int elev=0;
	float colorrange=255;
	gboolean  invccnt =FALSE,invrcnt=TRUE;

	FrameCenter.x=map->pixmap_size.x/2; 
	FrameCenter.y=map->pixmap_size.y/2;

	// set count direction, this step is required so that the image 
	// is always drawn from back to front
	if (xx >90 ) {invccnt =FALSE;invrcnt=FALSE;}
	if (xx > 180){invccnt =TRUE; invrcnt=FALSE;}
	if (xx > 270){invccnt =TRUE; invrcnt= TRUE;}

	if (zz > 0  && zz < 180  ) 
	{	// swap the invccnt from true to FALSE or vice versa
		invccnt = (invccnt == TRUE) ? FALSE : TRUE;
	}
	if (yy > 180) 
	{
		// swap the invrcnt from true to FALSE or vice versa
		invrcnt = (invrcnt == TRUE) ? FALSE : TRUE;
	}

	//set initial values for  rotation
	cAx=cos(xx*.01745); cAy=cos(yy*.01745); cAz=cos(zz*.01745);
	sAx=sin(xx*.01745); sAy=sin(yy*.01745); sAz=sin(zz*.01745);
	columns= col_list_ptr[0].lsize;
	rows= col_list_ptr[0].ptr[0].z;
	
	//find color range for brushes
	colorrange = (49/(map->MaxElev-map->minelev))*map->spatial_resolution_z;

	// test bondaries
	if(LeftOffset >= columns-Width)
	  LeftOffset = columns-Width;
	if (TopOffset >= rows-Height)
	  TopOffset = rows-Height;
	if(LeftOffset <10)
	  LeftOffset = 10;

	// set the number of rows and columns
	MeshOffset.x=LeftOffset+FrameWidth/2;
	MeshOffset.y=TopOffset+FrameHeight/2;
	columns= LeftOffset+Width;
	rows= col_list_ptr[0].ptr[0].z - TopOffset;
	TopOffset= col_list_ptr[0].ptr[0].z-(TopOffset+Height);

	// clear draw area area
	gdk_draw_rectangle (drawable,
		        GetPen(drawable,264),
			TRUE,
		        0,0,
		        map->pixmap_size.x,
			map->pixmap_size.y);

	// set initial value of the column count
	if (invccnt)
		col_count=columns-1; // for count up
	else
		col_count=LeftOffset; // for count down
		

	/* Start of drawing map drawing loop */
	while(!done)
	{
		if(col_count)
		{
			if (invrcnt)
				row_count= rows-1;
			else 
				row_count=TopOffset;

			index=0;
			while (row_count>=TopOffset && row_count < rows )
			{
				pos.x = col_count- MeshOffset.x;
				pos.y = MapRowPos(map,col_count,row_count)-MeshOffset.y;
				elev=MapElev(map,col_count,row_count); 
				pos=Rotation(map,pos.x,pos.y,elev,cAx,cAy,cAz,sAx,sAy,sAz);
				pos.x = pos.x*zoom+FrameCenter.x;  
				pos.y= pos.y*zoom+FrameCenter.y;
				//printf("x: %d y: %d",pos.x,pos.y);
				

				// Check to make sure that the points are on the map
				if(col_count > LeftOffset 
					&& row_count > TopOffset 
					&& row_count+Step < rows
					&& row_count+Step < col_list_ptr[col_count].lsize 
				        && col_count+Step < columns
					&& col_count+Step < col_list_ptr[0].lsize-5) 
				{
					// set to be wire frame or a solid model
					if(FrameMode && !MapColors)
					{
					  // set the color brush for this elevation
					  gc_com = (int)(elev* colorrange);
                                          
					  if(WaterLevel > 0 )
					    gc_com++;

					  if( elev < WaterLevel)
					    gc=GetPen(drawable,gc_com+50);
					  else
					    gc=GetPen(drawable,gc_com+ColorStartPosition);
						  
					}
					else if(!MapColors)
				        {
					    gc=GetPen(drawable,257);
					}
					else
					{
					  if(WaterLevel > 0 ) gc_com=1;else gc_com=0;

					  if( elev < WaterLevel)
					    gc = GetPen(drawable,MapColor(map,col_count,row_count)+50+gc_com);
					  else
					    gc = GetPen(drawable,MapColor(map,col_count,row_count)+
							ColorStartPosition+gc_com);
					}
					  
					
					// set the bondaries of the Polygon
					pt[1].x=LastPos[index-1].x; pt[1].y=LastPos[index-1].y;
					pt[2].x=pos.x; pt[2].y=pos.y;
					pt[3].x=LastPos[index].x; pt[3].y=LastPos[index].y;
					// Draw Polygon

					gdk_draw_polygon(drawable,gc,FrameMode,pt,4);    				
				}
				// keep points for next Polygon
				pt[0].x=LastPos[index].x;
				pt[0].y=LastPos[index].y;

				// keep points for next Polygon and column
				LastPos[index].y=pos.y;
				LastPos[index].x=pos.x;

				// set for next row
				index++;

				// set direction of the row count
				if (invrcnt)
				{
					pos.y=pos.y-Step;
					row_count=row_count-Step; // count down
				}
				else
				{
					pos.y=pos.y+Step;
					row_count=row_count+Step; // count up
				}

			}
		}
		
		// check for last column
		if (col_count < LeftOffset || (!invccnt && col_count+Step >= columns))
			done =TRUE;
		else 
		{
			// sets next column and the direction of the column count
			if (invccnt)
			{
				col_count=col_count-Step; // count down
			}
			else
			{
				col_count=col_count+Step; // count up
			}
		}
	}
}

/******************************************************************************
 * Name:         Initilize_map
 *-----------------------------------------------------------------------------
 * Description: Initialize map colors 
 *  
 *-----------------------------------------------------------------------------
 * Parameters:
 *    drawable- the GdkDrawable area
 ******************************************************************************/
void Initilize_map(GdkDrawable *drawable)
{
	static GdkColor gdk_color;

	/*set tool color and properties*/
	if (drawable != NULL)
	{
		gdk_color.red = (guint16) (1*65535.0);
		gdk_color.green = (guint16) (1*65535.0);
		gdk_color.blue = (guint16) (1*65535.0);
		gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
		newgc = gdk_gc_new(drawable);
		invertgc = gdk_gc_new(drawable);
		gdk_gc_set_foreground (newgc, &gdk_color);
		gdk_gc_set_foreground (invertgc,  &gdk_color);
		gdk_gc_set_fill (newgc, GDK_SOLID);
		gdk_gc_set_function (invertgc, GDK_INVERT);
	}
}

/******************************************************************************
 * Name:         GetPenNumber
 *-----------------------------------------------------------------------------
 * Description: Returns the pin index for a specified color (Actually a GdkGC).
 *              if a match is found in the color table. If it is not found the
 *              color is added to the color table and the new index values is 
 *              returned.
 *-----------------------------------------------------------------------------
 * Parameters:
 *    drawable- the GdkDrawable area
 * Returns:
 *    index to a color
 ******************************************************************************/
gint GetPenNumber(GdkGC *gc)
{
	GdkGCValues value, value2;
	gint i=0;
 
	gdk_gc_get_values(gc,&value);
	gdk_gc_get_values(GCTable[i],&value2);

	while (i <= maxpin && value.foreground.pixel != value2.foreground.pixel)
	{
		i++;
		gdk_gc_get_values(GCTable[i],&value2);
	}

	if (i > maxpin && maxpin < 1000)
	{
	  gdk_gc_copy(GCTable[i],gc); 
		maxpin++;
	}
 
	return(i);
}

/******************************************************************************
 * Name:         GetPen
 *-----------------------------------------------------------------------------
 * Description: creates the maps color table if it has not already been created
 *              and returns the color referance (Pin) pointed to by the color 
 *              index.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    drawable- the GdkDrawable area
 *    c -index to a color
 * Return:
 *    returns color (Actually a GdkGC)
 ******************************************************************************/
GdkGC *GetPen(GdkDrawable *drawable,int c)
{
	gdouble greencount = 0.30;
	gdouble redcount = 0.0;
	gdouble bluecount = 0.42;
	static GdkColor gdk_color;
	static gboolean pens_set = FALSE;
	int i=0;

 

	if (!pens_set)
	{
		/*set elevation colors*/

		for (i=1; i< 51; i++)
		{
			if (greencount <= 0.3)
				greencount = greencount + 0.01;
			else if (redcount <=0.6)
			{
				greencount = greencount + 0.01;
				redcount = redcount + 0.03;}
			else
			{
				bluecount = bluecount + 0.02;
				greencount = greencount + 0.02;
				redcount = redcount + 0.01;
			}

			if (bluecount >= 1) 
				bluecount = 1;
			if (redcount >= 1) 
				redcount = 1;
			if (greencount >= 1) 
				greencount = 1;

			gdk_color.red = (guint16) (redcount*65535.0);
			gdk_color.green = (guint16) (greencount*65535.0);
			gdk_color.blue = (guint16) (bluecount*65535.0);  
			gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
			GCTable[i] = gdk_gc_new(drawable);
			gdk_gc_set_foreground (GCTable[i], &gdk_color);
		}

		greencount = 0.0;
		redcount = 0.0;
		bluecount = 0.26;

		for (i=51; i< 100; i++)
		{
		        bluecount = bluecount + 0.015;
              		if (bluecount >= 1)
			{ 
			    bluecount = 1;
			    redcount = 1;
			    greencount = 1;
			}

			gdk_color.red = (guint16) (redcount*65535.0);
			gdk_color.green = (guint16) (greencount*65535.0);
			gdk_color.blue = (guint16) (bluecount*65535.0);  
			gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
			GCTable[i] = gdk_gc_new(drawable);
			gdk_gc_set_foreground (GCTable[i], &gdk_color);
		}

		greencount = 0.25;
		redcount = 0.0;
		bluecount = 0.0;

		for (i=100; i< 150; i++)
		{
		        greencount = greencount + 0.015;
              		if (bluecount >= 1)
			{ 
			    bluecount = 1;
			    redcount = 1;
			    greencount = 1;
			}

			gdk_color.red = (guint16) (redcount*65535.0);
			gdk_color.green = (guint16) (greencount*65535.0);
			gdk_color.blue = (guint16) (bluecount*65535.0);  
			gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
			GCTable[i] = gdk_gc_new(drawable);
			gdk_gc_set_foreground (GCTable[i], &gdk_color);
		}
		greencount = 0.0;
		redcount = 0.25;
		bluecount = 0.0;

		for (i=150; i< 200; i++)
		{
                        redcount = redcount + 0.015;
   
              		if (bluecount >= 1)
			{ 
			    bluecount = 1;
			    redcount = 1;
			    greencount = 1;
			}

			gdk_color.red = (guint16) (redcount*65535.0);
			gdk_color.green = (guint16) (greencount*65535.0);
			gdk_color.blue = (guint16) (bluecount*65535.0);  
			gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
			GCTable[i] = gdk_gc_new(drawable);
			gdk_gc_set_foreground (GCTable[i], &gdk_color);
		}
		greencount = 0.26;;
		redcount = 0.26;;
		bluecount = 0.26;;

		for (i=200; i< 1000; i++)
		{
		        greencount = greencount + 0.015;
                        redcount = redcount + 0.015;
                        bluecount = bluecount + 0.015;
              		if (bluecount >= 1)
			{ 
			    bluecount = 1;
			    redcount = 1;
			    greencount = 1;
			}

			gdk_color.red = (guint16) (redcount*65535.0);
			gdk_color.green = (guint16) (greencount*65535.0);
			gdk_color.blue = (guint16) (bluecount*65535.0);  
			gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
			GCTable[i] = gdk_gc_new(drawable);
			gdk_gc_set_foreground (GCTable[i], &gdk_color);
		}
		/* set water color */
		gdk_color.red = (guint16) (0.00*65535.0);
		gdk_color.green  = (guint16) (0.00*65535.0);
		gdk_color.blue  = (guint16) (0.72*65535.0);
		gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
		GCTable[0] = gdk_gc_new(drawable);
		gdk_gc_set_foreground (GCTable[0], &gdk_color);
    
		/* set water color*/
		gdk_color.red = (guint16) (0.00*65535.0);
		gdk_color.green = (guint16) (0.00*65535.0);
		gdk_color.blue = (guint16) (0.70*65535.0);
		gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
		GCTable[257] = gdk_gc_new(drawable);
		gdk_gc_set_foreground (GCTable[257], &gdk_color);

		/* set road color */
		gdk_color.red = (guint16) (0.70*65535.0);
		gdk_color.green = (guint16) (0.00*65535.0);
		gdk_color.blue = (guint16) (0.70*65535.0);
		gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
		GCTable[258] = gdk_gc_new(drawable);
		gdk_gc_set_foreground (GCTable[258], &gdk_color);

		/* set pipe color*/
		gdk_color.red = (guint16) (0.52*65535.0);
		gdk_color.green  = (guint16) (0.52*65535.0);
		gdk_color.blue = (guint16) (0.52*65535.0);
		gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
		GCTable[259] = gdk_gc_new(drawable);
		gdk_gc_set_foreground (GCTable[259], &gdk_color);

		/* set boundaries color */
		gdk_color.red = (guint16) (1.0*65535.0);
		gdk_color.green = (guint16) (1.0*65535.0);
		gdk_color.blue = (guint16) (0.36*65535.0);
		gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
		GCTable[260] = gdk_gc_new(drawable );
		gdk_gc_set_foreground (GCTable[260], &gdk_color);

		/* set Public color */
		gdk_color.red = (guint16) (1.0*65535.0);
		gdk_color.green = (guint16) (0.73*65535.0);
		gdk_color.blue = (guint16) (0.33*65535.0);
		gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
		GCTable[261] = gdk_gc_new(drawable);
		gdk_gc_set_foreground (GCTable[261], &gdk_color);

		/* set hypsograpy color */
		gdk_color.red = (guint16) (1.0*65535.0);
		gdk_color.green = (guint16) (0.77*65535.0);
		gdk_color.blue = (guint16) (0.56*65535.0);
		gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
		GCTable[262] = gdk_gc_new(drawable);
		gdk_gc_set_foreground (GCTable[262], &gdk_color);

		/* set Others color */
		gdk_color.red = (guint16) (1.0*65535.0);
		gdk_color.green = (guint16) (0.33*65535.0);
		gdk_color.blue = (guint16) (0.33*65535.0);
		gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
		GCTable[263] = gdk_gc_new(drawable);
		gdk_gc_set_foreground (GCTable[263], &gdk_color);

		/* set color black */
		gdk_color.red = (guint16) (0);
		gdk_color.green = (guint16) (0);
		gdk_color.blue = (guint16) (0);
		gdk_color_alloc (gdk_colormap_get_system(), &gdk_color);
		GCTable[264] = gdk_gc_new(drawable);
		gdk_gc_set_foreground (GCTable[263], &gdk_color);
 
		maxpin=264;

		pens_set = TRUE;
	}
 
	return (GCTable[c]);
}

/******************************************************************************
 * Name:         map_get_Water_level
 *-----------------------------------------------------------------------------
 * Description: Get the level below which the blue color range is used 
 *              to draw the map.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    ElevMap - Elevation map
 * returns:
 *    water- water level    
 ******************************************************************************/
gint  map_get_water_level(ElevMap *map)
{
  gint water=0;

  if( map->spatial_resolution_z != 1 && WaterLevel > 0)
    water= (gint)(WaterLevel/10)+map->minelev;
  else if ( WaterLevel > 0)
    water=WaterLevel+map->minelev ;
  else
    water= WaterLevel;

  if(map->minelev < 1)
    water=water+1;
  if(water < map->minelev)
    water=map->minelev;
  if(water > map->MaxElev)
    water = map->MaxElev;
  return water;
}

/******************************************************************************
 * Name:         map_set_Water_level
 *-----------------------------------------------------------------------------
 * Description: Set the level below which the blue color range is used 
 *              to draw the map.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    ElevMap - Elevation map
 *    water- water level    
 ******************************************************************************/
void  map_set_water_level(ElevMap *map, gint water)
{ 
        if(map->minelev < 1)
	  water=water-1;

    	if( map->spatial_resolution_z != 1 && water > 0)
	  WaterLevel=(water-map->minelev)*10;
        else if (water > 0)
	  WaterLevel = water-map->minelev ;
	else
	  WaterLevel=water;

}

/******************************************************************************
 * Name:         map_get_elv
 *-----------------------------------------------------------------------------
 * Description: Set the color range to draw map with. (is the starting 
 *              position in the Maps color table)
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    ElevMap - Elevation map
 *    range-range of colos   
 ******************************************************************************/
void  map_set_color_range(ElevMap *map,gint range)
{
  ColorStartPosition = range;
}

/******************************************************************************
 * Name:         map_get_elv
 *-----------------------------------------------------------------------------
 * Description: Returns the elevation at a specified location on the a 
 *              Elevation map
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    px - X poisiton of dem map
 *    py - Y poistion in 
 * Returns
 *    elv- a elevation 
 ******************************************************************************/
gint map_get_elv(col_list *demmap, gint px,gint py)
{
	gint elv;
	gint start_row_count = demmap[px].lsize-1;
	gint start_pyoff= demmap[px].row_offset-start_row_count;

	elv=demmap[px].ptr[demmap[px].lsize-(py-start_pyoff+1)].z;
	return (elv);
}

/******************************************************************************
 * Name:         map_get_pin
 *-----------------------------------------------------------------------------
 * Description: Returns a color index (pin) at a specified location on the a 
 *              Elevation map
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    px - X poisiton of dem map
 *    py - Y poistion in 
 * Returns
 *    gcreplace- color index (pin) 
 ******************************************************************************/
gint map_get_pin(col_list *demmap, gint px,gint py)
{
	gint gcreplace;
	gint start_row_count = demmap[px].lsize-1;
	gint start_pyoff= demmap[px].row_offset-start_row_count;

	gcreplace=demmap[px].ptr[demmap[px].lsize-(py-start_pyoff+1)].gc;
	return (gcreplace);
}

/******************************************************************************
 * Name:         gtk_copy_gc_Map
 *-----------------------------------------------------------------------------
 * Description: Returns a copy of the color (Actually a GdkGC) at a specified 
 *              location on the Elevation map
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    gc - a pointer to the varable to hold the color (Actually a GdkGC)
 *    px - X poisiton of dem map
 *    py - Y poistion in 
 ******************************************************************************/
void gtk_copy_gc_Map(col_list *demmap,GdkGC *gc,gint x,gint y)
{ 
	if (x > 0  && x < demmap[0].lsize-1)
	{
		gint start_row_count = demmap[x].lsize-1;
		gint start_pyoff= demmap[x].row_offset-start_row_count;
		if (y-start_pyoff >= 0 && y-start_pyoff < demmap[x].lsize-1)
			gdk_gc_copy(gc,GCTable[map_get_pin(demmap,x,y)]);
	}
}

/******************************************************************************
 * Name:         map_check_pin
 *-----------------------------------------------------------------------------
 * Description: Check if a color index (pin) value matches the one at a  
 *              specified location on the Elevation map
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    gcreplace - a color index value to test
 *    px - X poisiton of dem map
 *    py - Y poistion in 
 * Return:
 *    True if match else False
 ******************************************************************************/
gboolean map_check_pin(col_list *demmap,gint gcreplace,gint px,gint py)
{
	return (demmap[px].ptr[demmap[px].lsize-(py+1)].gc==gcreplace ||
		demmap[px].ptr[demmap[px].lsize-(py+1)].gc==-1);
}


/*-------------------- Map Drawing  routines for map ------------------------ */

/******************************************************************************
 * Name:         map_draw_point
 *-----------------------------------------------------------------------------
 * Description: replace the color index (pin) at a specified location on the   
 *              Elevation map with the new value of gc
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    gc - a color index 
 *    px - X position of dem map
 *    py - Y position of dem map
 ******************************************************************************/
void  map_draw_point    (col_list *demmap,gint gc,gint px, gint py)  
{
	if (px > 0 && py > 0 && py < demmap[px].lsize-1 &&
      px < (demmap[0].lsize-1))
	{
		demmap[px].ptr[demmap[px].lsize-(py+1)].gc=gc;
	}
}

/******************************************************************************
 * Name:         gtk_map_draw_point
 *-----------------------------------------------------------------------------
 * Description: replace the color (Actually a GdkGC) at a specified location    
 *              on the Elevation map with the new value of gc1
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    map - pointer to a Elevation map 
 *    gc1 - a color (Actually a GdkGC)
 *    px - X position of dem map
 *    py - Y position of dem map
 ******************************************************************************/
void  gtk_map_draw_point   (ElevMap *map,GdkGC *gc1,gint px, gint py)  
{
	gint gc=0;
	col_list *demmap=map->col_list_ptr;
	gint start_row_count = demmap[px].lsize-1;
	gint start_pyoff= demmap[px].row_offset-start_row_count;
 
	gc =GetPenNumber(gc1);
	map_draw_point    (demmap,gc,px,py-start_pyoff);
}

/******************************************************************************
 * Name:         map_draw_line
 *-----------------------------------------------------------------------------
 * Description: draw a line between two points on the Elevation map using the  
 *              color index (pin) gc.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    gc - a color index 
 *    start_x - X starting position 
 *    start_y - Y starting position 
 *    end_x - X ending position
 *    end_y - Y ending position
 ******************************************************************************/
void  map_draw_line    (col_list *demmap, gint gc, gint start_x, gint start_y,
						gint end_x, gint end_y)
{
	gint x,y;
	gint xlength = abs(start_x - end_x);
	gint ylength = abs(start_y - end_y);
	gint start_row_count;
	gint start_pyoff;
	double k =0.0;

	if (xlength < ylength && start_y <= end_y )
	{
		k= (double)xlength/ylength;
		for (y=0; y<=ylength; y++)
		{
			x = (double)y*k;
			if (start_x < end_x)
			{
				start_row_count = demmap[start_x+x].lsize-1;
				start_pyoff= demmap[start_x+x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x+x,start_y+y-start_pyoff);
			}
			else 
			{
				start_row_count = demmap[start_x-x].lsize-1;
				start_pyoff= demmap[start_x-x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x-x,(start_y+y)-start_pyoff);
			}
		} 
	}
	else if (xlength < ylength && start_y > end_y )
	{
		k= (double)xlength/(double)ylength;
		for (y=0; y<=ylength; y++)
		{
			x = (double)y*k;
			if (start_x < end_x)
			{
				start_row_count = demmap[start_x+x].lsize-1;
				start_pyoff= demmap[start_x+x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x+x,(start_y-y)-start_pyoff);}
			else 
			{
				start_row_count = demmap[start_x-x].lsize-1;
				start_pyoff= demmap[start_x-x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x-x,(start_y-y)-start_pyoff);
			}
		} 
	}
	else if(xlength > ylength && start_y <= end_y )
	{
		k= (double)ylength/(double)xlength;
		for (x=0; x<=xlength; x++)
		{
			y = (double)x*k;
			if (start_x < end_x)
			{
				start_row_count = demmap[start_x+x].lsize-1;
				start_pyoff= demmap[start_x+x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x+x,(start_y+y)-start_pyoff);}
			else 
			{
				start_row_count = demmap[start_x-x].lsize-1;
				start_pyoff= demmap[start_x-x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x-x,(start_y+y)-start_pyoff);}
		}
	} 
	else  if(xlength > ylength && start_y > end_y )
	{
		k= (double)ylength/(double)xlength;
		for (x=0; x<=xlength; x++)
		{
			y = (double)x*k;
			if (start_x < end_x)
			{
				start_row_count = demmap[start_x+x].lsize-1;
				start_pyoff= demmap[start_x+x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x+x,(start_y-y)-start_pyoff);
			}
			else 
			{
				start_row_count = demmap[start_x-x].lsize-1;
				start_pyoff= demmap[start_x-x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x-x,(start_y-y)-start_pyoff);
			}  
		}
	}
	else if (start_y < end_y )
	{
		for (y=0; y<=ylength; y++)
		{
			x = y;
			if (start_x < end_x)
			{
				start_row_count = demmap[start_x+x].lsize-1;
				start_pyoff= demmap[start_x+x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x+x,start_y+y-start_pyoff);
			}
			else 
			{
				start_row_count = demmap[start_x-x].lsize-1;
				start_pyoff= demmap[start_x-x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x-x,start_y+y-start_pyoff);
			} 
		}
	}
	else 
	{
		for (y=0; y<=ylength; y++)
		{
			x = y;
			if (start_x < end_x)
			{
				start_row_count = demmap[start_x+x].lsize-1;
				start_pyoff= demmap[start_x+x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x+x,start_y-y-start_pyoff);
			}
			else
			{ 
				start_row_count = demmap[start_x-x].lsize-1;
				start_pyoff= demmap[start_x-x].row_offset-start_row_count;
				map_draw_point(demmap,gc,start_x-x,start_y-y-start_pyoff);
			} 
		}
	}
}

/******************************************************************************
 * Name:         gtk_map_draw_line
 *-----------------------------------------------------------------------------
 * Description: draw a line between two points on the Elevation map using the  
 *              color (Actually a GdkGC) gc1.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    map - pointer to a Elevation map 
 *    gc1 - a color (Actually a GdkGC) 
 *    start_x - X starting position 
 *    start_y - Y starting position 
 *    end_x - X ending position
 *    end_y - Y ending position
 ******************************************************************************/
void  gtk_map_draw_line    (ElevMap *map,GdkGC *gc1, gint start_x, 
							gint start_y,gint end_x, gint end_y)
{
	gint gc=0;
	gc =GetPenNumber(gc1);
 
	map_draw_line    (map->col_list_ptr,gc,start_x,start_y,end_x,end_y);
}

/******************************************************************************
 * Name:         gtk_map_draw_lines
 *-----------------------------------------------------------------------------
 * Description: draw a sequence of connecting lines on the Elevation map using  
 *              the color (Actually a GdkGC) gc1.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    map - pointer to a Elevation map 
 *    gc1 - a color (Actually a GdkGC) 
 *    points - sequence of X & Y position draw a line between.
 *    numberoflines - number of lines to draw
 ******************************************************************************/
void  gtk_map_draw_lines    (ElevMap *map,GdkGC *gc1,
							 GdkPoint *points,gint numberoflines )
{
	GdkPoint  firstp,nextp;
	gint gc=0, i=0;
	gc =GetPenNumber(gc1);
 
	firstp.x=points[0].x;
	firstp.y=points[0].y;
	nextp.x=points[1].x;
	nextp.y=points[1].y;
	for (i=2; i<=numberoflines; i++)
	{
		map_draw_line(map->col_list_ptr,gc,nextp.x, nextp.y,firstp.x,firstp.y);
		nextp.x=firstp.x;
		firstp.x=points[i].x;
		nextp.y=firstp.y;
		firstp.y=points[i].y;
	}
}

/******************************************************************************
 * Name:         gtk_map_draw_rectangle
 *-----------------------------------------------------------------------------
 * Description: draw a rectangle on the Elevation map using the color   
 *              (Actually a GdkGC) gc1.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    map - pointer to a Elevation map 
 *    gc1 - a color (Actually a GdkGC) 
 *    fill - a flag that tells the if the rectangle is to be filled with the 
 *           gc1 color.
 *    start_x - X starting position 
 *    start_y - Y starting position 
 *    rec_width - width of rectangle
 *    rec_height - height of rectangle
 ******************************************************************************/
void  gtk_map_draw_rectangle (ElevMap *map,GdkGC *gc1,gboolean fill,
							  gint start_x,gint start_y, gint rec_width, 
							  gint rec_height)
{
	gint i=0,gc=0;
	gc =GetPenNumber(gc1);


	if (!fill)
	{
		map_draw_line    (map->col_list_ptr, gc, start_x, start_y,
			start_x+rec_width, start_y);
		map_draw_line    (map->col_list_ptr, gc, start_x, start_y,
			start_x, start_y+rec_height);
		map_draw_line    (map->col_list_ptr, gc, start_x+rec_width, start_y,
			start_x+rec_width, start_y+rec_height);
		map_draw_line    (map->col_list_ptr, gc, start_x, start_y+rec_height,
    		      start_x+rec_width, start_y+rec_height);
	}
	else 
	{
		for(i=start_y; i<= start_y+rec_height; i++)
			map_draw_line    (map->col_list_ptr, gc, start_x, i,
			start_x+rec_width, i);
	}
}


/******************************************************************************
 * Name:         map_filler
 *-----------------------------------------------------------------------------
 * Description: defines a box that bounds an area to start a recursive fill
 *              operation. The operation fill an area with in the box that
 *              is outline by the same color index gc starting from position px
 *              and py.
 *
 *              Note: This function called recursively with in it self.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    gc - a color index 
 *    px - X position of dem map
 *    py - Y position of dem map
 *    fillbox_top - top position of the fill box
 *    fillbox_bottom - bottom position of the fill box 
 *    fillbox_left - left position of the fill box
 *    fillbox_right - right position of the fill box
 ******************************************************************************/
void  map_filler    (col_list *demmap,gint gc,gint px, gint py,
		     gint fillbox_top,gint fillbox_bottom,
		     gint fillbox_left,gint fillbox_right)  
{
	gint start_row_count = demmap[px].lsize-1;
	gint start_pyoff= demmap[px].row_offset-start_row_count;
 
 

	if (px > 0 && py-start_pyoff >= 0 && py-start_pyoff < demmap[px].lsize-1 &&
		px < demmap[0].lsize-1)
	{
		if ( demmap[px].lsize-1 == py-start_pyoff || py-start_pyoff == 0 ||  
			demmap[0].lsize-1 == px  || px == 1 )
			map_draw_point    (demmap,gc,px,py-start_pyoff);
		else if (fillbox_top == py || fillbox_bottom == py || 
			     fillbox_left == px || fillbox_right == px )
			map_draw_point    (demmap,-1,px,py-start_pyoff); 
		else
		{
			map_draw_point    (demmap,gc,px,py-start_pyoff);  
			if (px > 0 && px < demmap[0].lsize-1)
			{
				start_row_count = demmap[px].lsize-1;
				start_pyoff= demmap[px].row_offset-start_row_count;
				if (py-start_pyoff+1 > 0 && py-start_pyoff+1 < demmap[px].lsize-1)
				{
					if( demmap[px].ptr[demmap[px].lsize-(py-start_pyoff+2)].gc!=gc)
						map_filler(demmap,gc,px,py+1,fillbox_top,fillbox_bottom,
									fillbox_left,fillbox_right);
				}
			}
 
			if ((px-1) > 0 && (px-1) < demmap[0].lsize-1)
			{
				start_row_count = demmap[px-1].lsize-1;
				start_pyoff= demmap[px-1].row_offset-start_row_count;
				if (py-start_pyoff > 0 &&  py < demmap[px-1].lsize-1)
				{
					if( demmap[px-1].ptr[demmap[px-1].lsize-(py+1-start_pyoff)].gc!=gc)
						map_filler(demmap,gc,px-1,py,fillbox_top,fillbox_bottom,
									fillbox_left,fillbox_right);
				}
			}
     
			if (px> 0 && px < demmap[0].lsize-1)
			{
				start_row_count = demmap[px].lsize-1;
				start_pyoff= demmap[px].row_offset-start_row_count;
				if ((py-1-start_pyoff) > 0 && (py-1-start_pyoff) < demmap[px].lsize-1)
				{
					if( demmap[px].ptr[demmap[px].lsize-(py-start_pyoff)].gc!=gc)
						map_filler(demmap,gc,px,py-1, fillbox_top,fillbox_bottom,
									fillbox_left,fillbox_right);
				}
			}
     
			if (px+1> 0 && px+1 < demmap[0].lsize-1)
			{
				start_row_count = demmap[px+1].lsize-1;
				start_pyoff= demmap[px+1].row_offset-start_row_count;
				if (py-start_pyoff > 0 && py-start_pyoff < demmap[px+1].lsize-1)
				{
					if( demmap[px+1].ptr[demmap[px+1].lsize-(py-start_pyoff+1)].gc!=gc)  
						map_filler(demmap,gc,px+1,py,fillbox_top,fillbox_bottom,
									fillbox_left,fillbox_right);
				}
			}
		}
	}
}

/******************************************************************************
 * Name:         map_replace 
 *-----------------------------------------------------------------------------
 * Description: defines a box that bounds an area to start a recursive fill
 *              operation. The operation fill an area with in the box in which
 *              all adjacent points have the same color index gc starting from 
 *              position px and py.
 *
 *              Note: This function called recursively with in it self.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    gc - a color index 
 *    px - X position of dem map
 *    py - Y position of dem map
 *    fillbox_top - top position of the fill box
 *    fillbox_bottom - bottom position of the fill box 
 *    fillbox_left - left position of the fill box
 *    fillbox_right - right position of the fill box
 ******************************************************************************/
void  map_replace    (col_list *demmap,gint gcpin,gint gcreplace,
					  gint px, gint py,
					  gint fillbox_top,gint fillbox_bottom,
					  gint fillbox_left,gint fillbox_right)  
{
	gint start_row_count = demmap[px].lsize-1;
	gint start_pyoff= demmap[px].row_offset-start_row_count;
 
	if (px > 0 && py-start_pyoff >= 0 && py-start_pyoff <= demmap[px].lsize-1 &&
		px < (demmap[0].lsize-1))
	{
		if ( (demmap[px].lsize-1 == py-start_pyoff || py-start_pyoff == 0 ||  
			demmap[0].lsize-1 == px  || px == 1)&& 
			map_check_pin(demmap,gcreplace,px,py-start_pyoff))
			map_draw_point    (demmap,gcpin,px,py-start_pyoff);
		else if (fillbox_top == py || fillbox_bottom == py || 
					fillbox_left == px || fillbox_right == px)
					map_draw_point    (demmap,-1,px,py-start_pyoff); 
		else if ( map_check_pin(demmap,gcreplace,px,py-start_pyoff))
		{
			map_draw_point    (demmap,gcpin,px,py-start_pyoff); 
   
			if (px > 0 && px < demmap[0].lsize-1)
			{
				start_row_count = demmap[px].lsize-1;
				start_pyoff= demmap[px].row_offset-start_row_count;
				if (py-start_pyoff+1 > 0 && py-start_pyoff+1 < demmap[px].lsize-1)
				{
					if( demmap[px].ptr[demmap[px].lsize-(py-start_pyoff+2)].gc!=gcpin)
						map_replace(demmap,gcpin,gcreplace,px,py+1,fillbox_top,fillbox_bottom,
									fillbox_left,fillbox_right);
				}
			} 
 
			if ((px-1) > 0 && (px-1) < demmap[0].lsize-1)
			{
				start_row_count = demmap[px-1].lsize-1;
				start_pyoff= demmap[px-1].row_offset-start_row_count;
				if (py-start_pyoff > 0 &&  py-start_pyoff < demmap[px-1].lsize-1)
				{
					if( demmap[px-1].ptr[demmap[px-1].lsize-(py+1-start_pyoff)].gc!=gcpin)
						map_replace(demmap,gcpin,gcreplace,px-1,py,fillbox_top,fillbox_bottom,
									fillbox_left,fillbox_right);
				}
			}
			
			if (px> 0 && px < demmap[0].lsize-1)
			{
				start_row_count = demmap[px].lsize-1;
				start_pyoff= demmap[px].row_offset-start_row_count;
				if ((py-1-start_pyoff) > 0 && (py-1-start_pyoff) < demmap[px].lsize-1)
				{
					if( demmap[px].ptr[demmap[px].lsize-(py-start_pyoff)].gc!=gcpin)
						map_replace(demmap,gcpin,gcreplace,px,py-1,fillbox_top,fillbox_bottom,
									fillbox_left,fillbox_right);
				}
			}

			if (px+1> 0 && px+1 < demmap[0].lsize-1)
			{
				start_row_count = demmap[px+1].lsize-1;
				start_pyoff= demmap[px+1].row_offset-start_row_count;
				if (py-start_pyoff > 0 && py-start_pyoff < demmap[px+1].lsize-1)
				{
					if( demmap[px+1].ptr[demmap[px+1].lsize-(py-start_pyoff+1)].gc!=gcpin)
						map_replace(demmap,gcpin,gcreplace,px+1,py,fillbox_top,fillbox_bottom,
						fillbox_left,fillbox_right);
				}
			}
		}
	}
}

/******************************************************************************
 * Name:         map_fill_bounded
 *-----------------------------------------------------------------------------
 * Description: The operation fill an area with in Elevation map that is 
 *              outline by the same color index gc starting from position px
 *              and py.
 *
 *              Note: This function is called recursively with in it self.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    gc - a color index 
 *    px - X position of dem map
 *    py - Y position of dem map
 ******************************************************************************/
void  map_fill_bounded    (col_list *demmap,gint gc,gint px, gint py) 
{  
	gint fillbox_top = py+110;
	gint fillbox_bottom = py-110;
	gint fillbox_left = px+110;
	gint fillbox_right = px-110;
	gint start_row_count;
	gint start_pyoff;
	gint i=0;
  
 
  
	/*begin fill */
	map_filler    (demmap,gc,px,py,fillbox_top,fillbox_bottom,
		fillbox_left,fillbox_right); 

	/*check rest of map for areas to fill*/
 
	for (i=fillbox_right; i<= fillbox_left; i++)
	{
		start_row_count = demmap[i].lsize-1;
		start_pyoff= demmap[i].row_offset-start_row_count;
		if (demmap[0].lsize-1 >= i && 1 <= i && 
			fillbox_top-start_pyoff < demmap[i].lsize-1 && 
			fillbox_bottom-start_pyoff >= 0)
		{
			if (demmap[i].ptr[demmap[i].lsize-(fillbox_top-start_pyoff+1)].gc == -1)
				map_fill_bounded (demmap,gc,i,fillbox_top);
		}
	}

	for (i=fillbox_right; i<= fillbox_left; i++)
	{
		start_row_count = demmap[i].lsize-1;
		start_pyoff= demmap[i].row_offset-start_row_count;
		if (demmap[0].lsize-1  >= i && 1 <= i && 
			fillbox_top-start_pyoff < demmap[i].lsize-1 && 
			fillbox_bottom-start_pyoff >= 0)
		{
			if (demmap[i].ptr[demmap[i].lsize-
		       (fillbox_bottom-start_pyoff+1)].gc == -1)
			   map_fill_bounded (demmap,gc,i,fillbox_bottom);
		}
	}

	start_row_count = demmap[fillbox_left].lsize-1;
	start_pyoff= demmap[fillbox_left].row_offset-start_row_count;
	for (i=fillbox_bottom; i<= fillbox_top; i++)
	{
		if (demmap[0].lsize-1 >= fillbox_left && 1 <= fillbox_right && 
			i-start_pyoff < demmap[fillbox_left].lsize-1 && i-start_pyoff >= 0)
		{
			if (demmap[fillbox_left].ptr[demmap[fillbox_left].lsize-
				(i+1-start_pyoff)].gc == -1)
				map_fill_bounded (demmap,gc,fillbox_left,i);
		}
	}

	start_row_count = demmap[fillbox_right].lsize-1;
	start_pyoff= demmap[fillbox_right].row_offset-start_row_count;
	for (i=fillbox_bottom; i<= fillbox_top; i++)
	{
		if (demmap[0].lsize-1 >= fillbox_left && 1 <= fillbox_right && 
			i-start_pyoff < demmap[fillbox_right].lsize-1 && i-start_pyoff >= 0)
		{
			if (demmap[fillbox_right].ptr[demmap[fillbox_right].lsize-
				(i+1-start_pyoff)].gc == -1)
				map_fill_bounded (demmap,gc,fillbox_right,i);
		}
	}
}

/******************************************************************************
 * Name:         gtk_map_fill_bounded
 *-----------------------------------------------------------------------------
 * Description: The operation fill an area with in Elevation map that is 
 *              outline by the same color gc starting from position px
 *              and py.
 *
 *              Note: This function calls map_fill_bounded.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    map - pointer to the elevation data in a Elevation map 
 *    gc1 - a color (Actually a GdkGC) 
 *    px - X position of dem map
 *    py - Y position of dem map
 ******************************************************************************/
void  gtk_map_fill_bounded    (ElevMap *map,GdkGC *gc1,gint px, gint py) 
{
	gint gc=0;
	col_list *demmap=map->col_list_ptr;
	gc =GetPenNumber(gc1);
  

	if (px > 0 && px < demmap[0].lsize-1)
	{
		gint start_row_count = demmap[px].lsize-1;
		gint start_pyoff= demmap[px].row_offset-start_row_count;
		if (py-start_pyoff >= 0 && py-start_pyoff)
			map_fill_bounded(demmap,gc,px,py); 
	}
}

/******************************************************************************
 * Name:         map_fill_replacer
 *-----------------------------------------------------------------------------
 * Description: The operation fills an area with in an Elevation map in which
 *              all adjacent points have the same color index gc starting from 
 *              position px and py.
 *
 *              Note: This function is called recursively with in it self.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    gc - a color index 
 *    px - X position of dem map
 *    py - Y position of dem map
 ******************************************************************************/
void  map_fill_replacer (col_list *demmap,gint gc,gint gcreplace,
						 gint px, gint py) 
{
	gint fillbox_top = py+110;
	gint fillbox_bottom = py-110;
	gint fillbox_left = px+110;
	gint fillbox_right = px-110;
	gint start_row_count;
	gint start_pyoff;
	gint i=0;

 
	map_replace (demmap,gc,gcreplace,px,py,fillbox_top,fillbox_bottom,
				 fillbox_left,fillbox_right); 

	/*check rest of map for areas to fill*/
 
	for (i=fillbox_right; i<= fillbox_left; i++)
	{
		start_row_count = demmap[i].lsize-1;
		start_pyoff= demmap[i].row_offset-start_row_count;
		if (demmap[0].lsize-1 >= i && 1 <= i && 
			fillbox_top-start_pyoff < demmap[i].lsize-1 && 
			fillbox_bottom-start_pyoff >= 0)
		{
			if (demmap[i].ptr[demmap[i].lsize-(fillbox_top-start_pyoff+1)].gc == -1)
				map_fill_replacer (demmap,gc,gcreplace,i,fillbox_top);
		}
	}

	for (i=fillbox_right; i<= fillbox_left; i++)
	{
		start_row_count = demmap[i].lsize-1;
		start_pyoff= demmap[i].row_offset-start_row_count;
		if (demmap[0].lsize-1  >= i && 1 <= i && 
			fillbox_top-start_pyoff < demmap[i].lsize-1 && 
			fillbox_bottom-start_pyoff >= 0)
		{
			if (demmap[i].ptr[demmap[i].lsize-
				(fillbox_bottom-start_pyoff+1)].gc == -1)
				map_fill_replacer (demmap,gc,gcreplace,i,fillbox_bottom);
		}
	}

	start_row_count = demmap[fillbox_left].lsize-1;
	start_pyoff= demmap[fillbox_left].row_offset-start_row_count;
	for (i=fillbox_bottom; i<= fillbox_top; i++)
	{
		if (demmap[0].lsize-1 >= fillbox_left && 1 <= fillbox_right && 
			i-start_pyoff < demmap[fillbox_left].lsize-1 && i-start_pyoff >= 0)
		{
			if (demmap[fillbox_left].ptr[demmap[fillbox_left].lsize-
				(i+1-start_pyoff)].gc == -1)
				map_fill_replacer (demmap,gc,gcreplace,fillbox_left,i);
		}
	}

	start_row_count = demmap[fillbox_right].lsize-1;
	start_pyoff= demmap[fillbox_right].row_offset-start_row_count;
	for (i=fillbox_bottom; i<= fillbox_top; i++)
	{
		if (demmap[0].lsize-1 >= fillbox_left && 1 <= fillbox_right && 
			i-start_pyoff < demmap[fillbox_right].lsize-1 && i-start_pyoff >= 0)
		{
			if (demmap[fillbox_right].ptr[demmap[fillbox_right].lsize-
				(i+1-start_pyoff)].gc == -1)
				map_fill_replacer (demmap,gc,gcreplace,fillbox_right,i);
		}
	}
}

/******************************************************************************
 * Name:         gtk_map_fill_replace
 *-----------------------------------------------------------------------------
 * Description: The operation fills an area with in an Elevation map in which
 *              all adjacent points have the same color gc1 starting from 
 *              position px and py.
 *
 *              Note: This function is calls map_fill_replacer.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    map - pointer to the elevation data in a Elevation map 
 *    gc1 - a color (Actually a GdkGC) 
 *    px - X position of dem map
 *    py - Y position of dem map
 ******************************************************************************/
void  gtk_map_fill_replace (ElevMap *map,GdkGC *gc1,gint px, gint py) 
{
	gint gcreplace,gc=0;
	col_list *demmap=map->col_list_ptr;
	gc =GetPenNumber(gc1);


	/*begin fill replace */
	if (px > 0  && px < demmap[0].lsize-1)
	{
		gint start_row_count = demmap[px].lsize-1;
		gint start_pyoff= demmap[px].row_offset-start_row_count;
		if (py-start_pyoff >= 0 && py-start_pyoff < demmap[px].lsize-1)
		{
			gcreplace = map_get_pin(demmap, px, py);
			map_fill_replacer (demmap,gc,gcreplace,px,py); 
		}
	} 
}

/******************************************************************************
 * Name:         map_draw_elv
 *-----------------------------------------------------------------------------
 * Description: replace the elevation value at a specified location on the   
 *              Elevation map with the new value of pz
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    px - X position of dem map
 *    py - Y position of dem map
 *    pz - elevation value
 ******************************************************************************/
void  map_draw_elv    (col_list *demmap, gint px,gint py, gint pz)  
{
	if (px > 0 && py > 0 && py < demmap[px].lsize-1 &&
		px < (demmap[0].lsize-1))
	{
		demmap[px].ptr[demmap[px].lsize-(py+1)].z=pz;
	}
  
}



/*-------------------- Map DXF drawing routines for map----------------------- */
/* the following code is used to create elevation maps from DXF frames. It is  *
 * still in development and should not be used at this time                    */                           

void  map_draw_fline    (col_list *demmap, gint gc, 
			 gint start_x, gint start_y, gint start_z, 
			 gint end_x, gint end_y, gint end_z)
{
 float  x,y,z;
 gint xlength = abs(start_x - end_x);
 gint ylength = abs(start_y - end_y);
 gint start_row_count;
 gint start_pyoff;
 double k =0.0;

 
 if (xlength < ylength && start_y <= end_y ){
   k= (double)xlength/ylength;
   for (y=0; y<=ylength; y=y+.5){
     x = (double)y*k;
     if (start_x < end_x){//0k
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x+x,start_y+y-start_pyoff);
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_elv(demmap,start_x+x,start_y+y-start_pyoff,z);}
     else {//ok
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x-x,(start_y+y)-start_pyoff);
        if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_elv(demmap,start_x-x,(start_y+y)-start_pyoff,z);}
   } 
 }
 else if (xlength < ylength && start_y > end_y ){
   k= (double)xlength/(double)ylength;
   for (y=0; y<=ylength; y=y+.5){
     x = (double)y*k;
     if (start_x < end_x){//ok
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x+x,(start_y-y)-start_pyoff);
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_elv(demmap,start_x+x,(start_y-y)-start_pyoff,z);}
     else {//ok*
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x-x,(start_y-y)-start_pyoff);
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_elv(demmap,start_x-x,(start_y-y)-start_pyoff,z);}
   } 
 }
 else if(xlength > ylength && start_y <= end_y ){
   k= (double)ylength/(double)xlength;
   for (x=0; x<=xlength; x=x+.5){
     y = (double)x*k;
     if (start_x < end_x){//0k
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x+x,(start_y+y)-start_pyoff);
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/xlength)*x)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/xlength)*(xlength-x))+end_z;	
       else z=start_z; 
       map_draw_elv(demmap,start_x+x,(start_y+y)-start_pyoff,z);}
     else {//ok
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x-x,(start_y+y)-start_pyoff);
      if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/xlength)*x)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/xlength)*(xlength-x))+end_z;	
       else z=start_z;
       map_draw_elv(demmap,start_x-x,(start_y+y)-start_pyoff,z);}
   }
 } 
 else  if(xlength > ylength && start_y > end_y ){
   k= (double)ylength/(double)xlength;
   for (x=0; x<=xlength; x=x+.5){
     y = (double)x*k;
     if (start_x < end_x){//ok
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x+x,(start_y-y)-start_pyoff);
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/xlength)*x)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/xlength)*(xlength-x))+end_z;	
       else z=start_z;
       map_draw_elv(demmap,start_x+x,(start_y-y)-start_pyoff,z);}
     else {//ok
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x-x,(start_y-y)-start_pyoff);
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/xlength)*x)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/xlength)*(xlength-x))+end_z;	
       else z=start_z;
       map_draw_elv(demmap,start_x-x,(start_y-y)-start_pyoff,z);}  
   }
 }
 else if (start_y < end_y ){
   for (y=0; y<=ylength; y=y+.5){
     x = y;
     if (start_x < end_x){//ok
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x+x,start_y+y-start_pyoff);
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_elv(demmap,start_x+x,start_y+y-start_pyoff,z);}
     else {//ok
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x-x,start_y+y-start_pyoff);
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_elv(demmap,start_x-x,start_y+y-start_pyoff,z);} 
   }
 }
 else {
   for (y=0; y<=ylength; y=y+.5){
     x = y;
     if (start_x < end_x){
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x+x,start_y-y-start_pyoff);
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_elv(demmap,start_x+x,start_y-y-start_pyoff,z);}
     else{ //ok
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       map_draw_point(demmap,gc,start_x-x,start_y-y-start_pyoff);
       if (start_z < end_z) 
	 z=(((float)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_elv(demmap,start_x-x,start_y-y-start_pyoff,z);} 
   }
 }
}



void  gtk_map_draw_fline    (ElevMap *map,GdkGC *gc1, 
			     gint start_x, gint start_y,gint start_z,
			     gint end_x, gint end_y, gint end_z)
{
 gint gc=0;
 gc =GetPenNumber(gc1);
 
 map_draw_fline    (map->col_list_ptr,gc,start_x,start_y,start_z,
		   end_x,end_y,end_z);
}




/******************************************************************************
 * Name:         map_fill_face
 *-----------------------------------------------------------------------------
 * Description: defines a box that bounds an area to start a recursive fill
 *              operation. The operation fill an area with in the box that
 *              is outline by the same color index gc  starting from position px
 *              and py. This fill fills a color and elevation value.
 *
 *              Note: This function called recursively with in it self.
 *
 *-----------------------------------------------------------------------------
 * Parameters:
 *    demmap - pointer to the elevation data in a Elevation map 
 *    gc - a color index 
 *    px - X position of dem map
 *    py - Y position of dem map
 *    pz - Z elvation of point on dem map
 *    xslope - slope of the x plain
 *    Yslope - Slope of the y plain
 *    fillbox_top - top position of the fill box
 *    fillbox_bottom - bottom position of the fill box 
 *    fillbox_left - left position of the fill box
 *    fillbox_right - right position of the fill box
 ******************************************************************************/
void  map_fill_face1 (col_list *demmap,gint gc,gint px, gint py, gint pz,
		      double xslope, double  Yslope,
		      gint fillbox_top,gint fillbox_bottom,
		      gint fillbox_left,gint fillbox_right)  
		
{
	gint start_row_count = demmap[px].lsize-1;
	gint start_pyoff= demmap[px].row_offset-start_row_count;
 
 

	if (px > 0 && py-start_pyoff >= 0 && py-start_pyoff < demmap[px].lsize-1 &&
		px < demmap[0].lsize-1)
	{
		if ( demmap[px].lsize-1 == py-start_pyoff || py-start_pyoff == 0 ||  
			demmap[0].lsize-1 == px  || px == 1 )
			map_draw_point    (demmap,gc,px,py-start_pyoff);
		else if (fillbox_top == py || fillbox_bottom == py || 
			     fillbox_left == px || fillbox_right == px )
			map_draw_point    (demmap,-1,px,py-start_pyoff); 
		else
		{
			map_draw_point    (demmap,gc,px,py-start_pyoff);  
			if (px > 0 && px < demmap[0].lsize-1)
			{
				start_row_count = demmap[px].lsize-1;
				start_pyoff= demmap[px].row_offset-start_row_count;
				if (py-start_pyoff+1 > 0 && py-start_pyoff+1 < demmap[px].lsize-1)
				{
					if( demmap[px].ptr[demmap[px].lsize-(py-start_pyoff+2)].gc!=gc)
						map_filler(demmap,gc,px,py+1,fillbox_top,fillbox_bottom,
									fillbox_left,fillbox_right);
				}
			}
 
			if ((px-1) > 0 && (px-1) < demmap[0].lsize-1)
			{
				start_row_count = demmap[px-1].lsize-1;
				start_pyoff= demmap[px-1].row_offset-start_row_count;
				if (py-start_pyoff > 0 &&  py < demmap[px-1].lsize-1)
				{
					if( demmap[px-1].ptr[demmap[px-1].lsize-(py+1-start_pyoff)].gc!=gc)
						map_filler(demmap,gc,px-1,py,fillbox_top,fillbox_bottom,
									fillbox_left,fillbox_right);
				}
			}
     
			if (px> 0 && px < demmap[0].lsize-1)
			{
				start_row_count = demmap[px].lsize-1;
				start_pyoff= demmap[px].row_offset-start_row_count;
				if ((py-1-start_pyoff) > 0 && (py-1-start_pyoff) < demmap[px].lsize-1)
				{
					if( demmap[px].ptr[demmap[px].lsize-(py-start_pyoff)].gc!=gc)
						map_filler(demmap,gc,px,py-1, fillbox_top,fillbox_bottom,
									fillbox_left,fillbox_right);
				}
			}
     
			if (px+1> 0 && px+1 < demmap[0].lsize-1)
			{
				start_row_count = demmap[px+1].lsize-1;
				start_pyoff= demmap[px+1].row_offset-start_row_count;
				if (py-start_pyoff > 0 && py-start_pyoff < demmap[px+1].lsize-1)
				{
					if( demmap[px+1].ptr[demmap[px+1].lsize-(py-start_pyoff+1)].gc!=gc)  
						map_filler(demmap,gc,px+1,py,fillbox_top,fillbox_bottom,
									fillbox_left,fillbox_right);
				}
			}
		}
	}
}


void  map_fill_face    (col_list *demmap,gint gc,gint x0,gint y0,gint z0,
			gint start_x,gint start_y,gint start_z,
			gint end_x,gint end_y, gint end_z)
		
/* ************************************************************
*   Interpolate all of the beginning (or ending) x,z endpoints 
* along the entire Y-span of a line segment.
************************************************************ */
{		       
 float  x,y,z;
 gint xlength = abs(start_x - end_x);
 gint ylength = abs(start_y - end_y);
 gint start_row_count;
 gint start_pyoff;
 double k =0.0;

 
 if (xlength < ylength && start_y <= end_y ){
   k= (double)xlength/ylength;
   for (y=0; y<=ylength; y++){
     x = (double)y*k;
     if (start_x < end_x){
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x+x,start_y+y-start_pyoff,z);}
     else {
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x-x,(start_y+y)-start_pyoff,z);}
   } 
 }
 else if (xlength < ylength && start_y > end_y ){
   k= (double)xlength/(double)ylength;
   for (y=0; y<=ylength; y++){
     x = (double)y*k;
     if (start_x < end_x){
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x+x,(start_y-y)-start_pyoff,z);}
     else {
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x-x,(start_y-y)-start_pyoff,z);}
   } 
 }
 else if(xlength > ylength && start_y <= end_y ){
   k= (double)ylength/(double)xlength;
   for (x=0; x<=xlength; x++){
     y = (double)x*k;
     if (start_x < end_x){
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/xlength)*x)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/xlength)*(xlength-x))+end_z;	
       else z=start_z; 
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x+x,(start_y+y)-start_pyoff,z);}
     else {
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/xlength)*x)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/xlength)*(xlength-x))+end_z;	
       else z=start_z;
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x-x,(start_y+y)-start_pyoff,z);}
   }
 } 
 else  if(xlength > ylength && start_y > end_y ){
   k= (double)ylength/(double)xlength;
   for (x=0; x<=xlength; x++){
     y = (double)x*k;
     if (start_x < end_x){
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/xlength)*x)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/xlength)*(xlength-x))+end_z;	
       else z=start_z;
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x+x,(start_y-y)-start_pyoff,z);}
     else {
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/xlength)*x)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/xlength)*(xlength-x))+end_z;	
       else z=start_z;
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x-x,(start_y-y)-start_pyoff,z);}  
   }
 }
 else if (start_y < end_y ){
   for (y=0; y<=ylength; y++){
     x = y;
     if (start_x < end_x){
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x+x,start_y+y-start_pyoff,z);}
     else {
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x-x,start_y+y-start_pyoff,z);} 
   }
 }
 else {
   for (y=0; y<=ylength; y++){
     x = y;
     if (start_x < end_x){
       start_row_count = demmap[(int)(start_x+x)].lsize-1;
       start_pyoff= demmap[(int)(start_x+x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x+x,start_y-y-start_pyoff,z);}
     else{ 
       start_row_count = demmap[(int)(start_x-x)].lsize-1;
       start_pyoff= demmap[(int)(start_x-x)].row_offset-start_row_count;
       if (start_z < end_z) 
	 z=(((double)(end_z-start_z)/ylength)*y)+start_z;
       else if (end_z < start_z) 
	 z=(((double)(start_z-end_z)/ylength)*(ylength-y))+end_z;	
       else z=start_z;
       map_draw_fline (demmap, gc,x0,y0,z0,
		       start_x-x,start_y-y-start_pyoff,z);} 
   }
 }
}


void  gtk_map_draw_face  (ElevMap *map,GdkGC *gc1,
			  float x0,float y0,float z0,
			  float x1,float y1,float z1, 
			  float x2,float y2,float z2, 
			  float x3,float y3,float z3)
{
 gint gc=0;
 gc =GetPenNumber(gc1);
 
  if (map->col_list_ptr != NULL){
   
    map_fill_face    (map->col_list_ptr, gc,
		      (x0*map->spatial_resolution_x)+5,
		      map->col_list_ptr[0].lsize-
		      (y0*map->spatial_resolution_y)-5,
		      (z0*map->spatial_resolution_z),
		      (x1*map->spatial_resolution_x)+5,
		      map->col_list_ptr[0].lsize-
		      (y1*map->spatial_resolution_y)-5,
		      (z1*map->spatial_resolution_z),
		      (x2*map->spatial_resolution_x)+5,
		      map->col_list_ptr[0].lsize-
		      (y2*map->spatial_resolution_y)-5,
		      (z2*map->spatial_resolution_z));
    map_fill_face    (map->col_list_ptr, gc,
		      (x1*map->spatial_resolution_x)+5,
		      map->col_list_ptr[0].lsize-
		      (y1*map->spatial_resolution_y)-5,
		      (z1*map->spatial_resolution_z),
		      (x0*map->spatial_resolution_x)+5,
		      map->col_list_ptr[0].lsize-
		      (y0*map->spatial_resolution_y)-5,
		      (z0*map->spatial_resolution_z),
		      (x3*map->spatial_resolution_x)+5,
		      map->col_list_ptr[0].lsize-
		      (y3*map->spatial_resolution_y)-5,
		      (z3*map->spatial_resolution_z));

    
  }
}
